pub struct SessionAudioConsumerFactoryProxy { /* private fields */ }Implementations
Sourceimpl SessionAudioConsumerFactoryProxy
impl SessionAudioConsumerFactoryProxy
Sourcepub fn new(channel: AsyncChannel) -> Self
pub fn new(channel: AsyncChannel) -> Self
Create a new Proxy for fuchsia.media/SessionAudioConsumerFactory.
Sourcepub fn take_event_stream(&self) -> SessionAudioConsumerFactoryEventStream
pub fn take_event_stream(&self) -> SessionAudioConsumerFactoryEventStream
Get a Stream of events from the remote end of the protocol.
Panics
Panics if the event stream was already taken.
Sourcepub fn create_audio_consumer(
&self,
session_id: u64,
audio_consumer_request: ServerEnd<AudioConsumerMarker>,
) -> Result<(), Error>
pub fn create_audio_consumer( &self, session_id: u64, audio_consumer_request: ServerEnd<AudioConsumerMarker>, ) -> Result<(), Error>
Creates an AudioConsumer, which is an interface for playing audio, bound
to a particular session. session_id is the identifier of the media session
for which audio is to be rendered.