pub struct Usage2AudioConsumerFactoryProxy { /* private fields */ }Implementations
Sourceimpl Usage2AudioConsumerFactoryProxy
impl Usage2AudioConsumerFactoryProxy
Sourcepub fn new(channel: AsyncChannel) -> Self
pub fn new(channel: AsyncChannel) -> Self
Create a new Proxy for fuchsia.media/Usage2AudioConsumerFactory.
Sourcepub fn take_event_stream(&self) -> Usage2AudioConsumerFactoryEventStream
pub fn take_event_stream(&self) -> Usage2AudioConsumerFactoryEventStream
Get a Stream of events from the remote end of the protocol.
Panics
Panics if the event stream was already taken.
Sourcepub fn create_audio_consumer(
&self,
usage: AudioRenderUsage2,
audio_consumer_request: ServerEnd<AudioConsumerMarker>,
) -> Result<(), Error>
pub fn create_audio_consumer( &self, usage: AudioRenderUsage2, audio_consumer_request: ServerEnd<AudioConsumerMarker>, ) -> Result<(), Error>
Creates an AudioConsumer, which is an interface for playing audio, given a usage value.
Audio submitted to such a consumer is always rendered locally.