pub struct StreamProcessorRemoveBufferRequest {
pub port: Option<Port>,
pub buffer_lifetime_ordinal: Option<u64>,
pub buffer_index: Option<u32>,
/* private fields */
}Fields
port: Option<Port>Must be set. Which port (input or output) the buffer is for.
buffer_lifetime_ordinal: Option<u64>Scoped to input or output, this is the buffer_lifetime_ordinal under which the buffer_index is meaningful.
buffer_index: Option<u32>The buffer_index of the buffer previously established (and most recently established) by AddBuffer, or by SetInputBufferPartialSettings / SetOutputBufferPartialSettings. The scope of this value is under input vs. output and under the buffer_lifetime_ordinal value.
When using dynamic buffers, the buffer_index values that are between AddBuffer starting and RemoveBuffer completing (the active buffer_index values) must be unique for a given Port value and buffer_lifetime_ordinal value.
Clients using dynamic buffers may wish to not re-use a buffer_index value quickly upon completion of RemoveBuffer just to make debugging/diagnosing easier, but technically re-use of buffer_index in a subsequent AddBuffer is permitted as soon as RemoveBuffer completes (from the StreamProcessor point of view - the client may have its own reasons to not re-use that quickly). AddBuffer under an old (not current) buffer_lifetime_ordinal is never allowed.